import datetime
from typing import Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="BlockReason")


@_attrs_define
class BlockReason:
    """Schema for client blocking reason.

    Example:
        {'blocked_by_uuid': '123e4567-e89b-12d3-a456-426614174000', 'expires_at': '2024-02-01T00:00:00Z', 'reason':
            'Spam messages'}

    Attributes:
        reason (str): Reason for blocking
        blocked_by_uuid (UUID): UUID of user who blocked the client
        expires_at (Union[None, Unset, datetime.datetime]): When block expires (optional)
    """

    reason: str
    blocked_by_uuid: UUID
    expires_at: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        reason = self.reason

        blocked_by_uuid = str(self.blocked_by_uuid)

        expires_at: Union[None, Unset, str]
        if isinstance(self.expires_at, Unset):
            expires_at = UNSET
        elif isinstance(self.expires_at, datetime.datetime):
            expires_at = self.expires_at.isoformat()
        else:
            expires_at = self.expires_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "reason": reason,
                "blocked_by_uuid": blocked_by_uuid,
            }
        )
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        reason = d.pop("reason")

        blocked_by_uuid = UUID(d.pop("blocked_by_uuid"))

        def _parse_expires_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                expires_at_type_0 = isoparse(data)

                return expires_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        expires_at = _parse_expires_at(d.pop("expires_at", UNSET))

        block_reason = cls(
            reason=reason,
            blocked_by_uuid=blocked_by_uuid,
            expires_at=expires_at,
        )

        block_reason.additional_properties = d
        return block_reason

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
